call plug#begin('$HOME/.cache/vim-plug')

" [] shortcuts
Plug 'tpope/vim-unimpaired'

" coc autocomplete engine
Plug 'neoclide/coc.nvim', {'branch': 'release'}

" Directory browser _ to open
Plug 'tpope/vim-vinegar'

" Fuzzy Finder
Plug '/usr/local/opt/fzf'
Plug 'junegunn/fzf.vim'

" Seeeearch with ag
Plug 'mileszs/ack.vim'

" Quoting
Plug 'tpope/vim-surround'

" Git helper
Plug 'tpope/vim-fugitive'

" Unix Convenience Commands/helpers
Plug 'tpope/vim-eunuch'

" Vimwiki
Plug 'vimwiki/vimwiki'

" Colorschemes
Plug 'morhetz/gruvbox'
Plug 'Lokaltog/vim-monotone'
Plug 'igrmk/kull-vim'
" removed?
"Plug 'rj-white/vim-colors-paramountblue'
Plug 'arzg/vim-mayanfog'
Plug 'cormacrelf/vim-colors-github'
Plug 'junegunn/seoul256.vim'

" Show indents for space-indents etc
"Plug 'Yggdroot/indentLine'
"Plug 'nathanaelkane/vim-indent-guides'

" Languages
" EditorConfig project config files
Plug 'editorconfig/editorconfig-vim'

" C/C++
"Plug 'Shougo/deoplete-clangx' " deoplete removed

" Go
Plug 'fatih/vim-go'

" Rust
Plug 'rust-lang/rust.vim'

" Beancount
Plug 'nathangrigg/vim-beancount'

" Yaml
Plug 'stephpy/vim-yaml'

" Python
Plug 'tmhedberg/SimpylFold'
"Plug 'zchee/deoplete-jedi' " deoplete removed

" Helm
Plug 'towolf/vim-helm'

call plug#end()

"""""""""""""""""""""""""""""""""""""""""" GENERAL SETTINGS
" load filetype specific indent files
" ~/.vim/indent/*.vim
filetype plugin indent on
"
" Enable syntax highlighting
syntax enable
"
" Set leader to space
let mapleader=" "

" Show when I type my leader (bottom right)
set showcmd

if has('nvim')
	set clipboard+=unnamedplus
endif

"""""""""""""""""""""""""""""""""""""""""" PLUGIN SETTINGS
" ack.vim use ag if available
"if executable('ag')
"  let g:ackprg = 'ag --vimgrep'
"endif

"if executable('rg')
"  set grepprg=rg\ --vimgrep\ --no-heading\ --smart-case
"  let g:ackprg = 'ag --vimgrep'
"endif

"""""""""""""""""""""""""""""""""""
" Folding
"""""""""""""""""""""""""""""""""""
" enable folding
set foldenable

" Keep most folds open
set foldlevelstart=10

" Fold based on indents
set foldmethod=syntax

"""""""""""""""""""""""""""""""""""
" Windows and Buffers
"""""""""""""""""""""""""""""""""""

" Buffer Management
" Confirm asks if you want to save before switching,
" but still gives the option
set confirm

" Don't resize my freaking splits PLZ
set noequalalways

" Diff splits always vertical
set diffopt=filler,vertical

"""""""""""""""""""""""""""""""""""""""""" KEYBINDS

" Move visual lines, in the case of wrapped text
nnoremap j gj
nnoremap k gk

" ESC is for chumps
imap jk <Esc>
"imap kj <Esc>

if has('nvim')
	" Escape nvim-only terminal
	tnoremap jk <C-\><C-n>
	"tnoremap kj <C-\><C-n>
	" Turn on nvim mouse
	set mouse=a
endif

" Stupid help key
inoremap <F1> <ESC>
nnoremap <F1> <ESC>
vnoremap <F1> <ESC>

" Window Nav
nmap <silent> <C-j> :wincmd j<CR>
nmap <silent> <C-k> :wincmd k<CR>
nmap <silent> <C-l> :wincmd l<CR>
nmap <silent> <C-h> :wincmd h<CR>
nmap <silent> <C-c> :wincmd c<CR>

if !has('nvim')
	" In terminal also!
	" (exits terminal insert mode, switches window)
	tmap <silent> <C-j> <C-\><C-n>:wincmd j<CR>
	tmap <silent> <C-k> <C-\><C-n>:wincmd k<CR>
	tmap <silent> <C-l> <C-\><C-n>:wincmd l<CR>
	tmap <silent> <C-h> <C-\><C-n>:wincmd h<CR>
endif

map ; :
"noremap ;; ;

""""""""""""""""""""""""""""""""""""""" Leader
" Toggle fold
nmap <leader>f za
nmap <leader>F zA

" fzf buffers... so fast (in theory)
nmap <leader>B :Buffers<CR>

" Switch PWD to file's directory
nnoremap <leader>cd :cd %:p:h<CR>:pwd<CR>

" Turn off search highlighting
nnoremap <leader>h :nohl<CR>

" insert timestamp
nnoremap <leader>t "=strftime("%c")<CR>P

" something something spellcheck ¯\_(ツ)_/¯
nnoremap <leader>s z=

" How about... writing the file and quitting!
nnoremap <leader>w :update<CR>
nnoremap <leader>x :xit<CR>
nnoremap <leader>q :q<CR>


"""""""""""""""""""""""""""""""""""""""" style stuff
" Use all the colors
" set termguicolors

" Colorscheme
colorscheme gruvbox
set background=dark

" limit highlighting to open big files
set synmaxcol=512

" font in GUI
set guifont=Monaco:h14

" Change terminal title
set title

" Show whitespace, invisible
set list
set listchars=tab:►-,trail:✗,nbsp:✗,extends:>,precedes:<

" Default to nowrap for code
set nowrap

" Line numbers -- you need to use these better
set number
set relativenumber

" highlight the line my cursor is in
set cursorline
set cursorcolumn

" highlight 80th column
set colorcolumn=80

" autocomplete visual menu
set wildmenu

" autocomplete configuration
set wildmode=longest:full,full

" autocomplete - case insensitivity
if exists("&wildignorecase")
    set wildignorecase
endif

" These two options change search. All lowercase = case insensitive, any upper
" = case sensitive
set ignorecase
set smartcase

" Only redraw when enscessary
set lazyredraw

" Highlight matching containers
set showmatch

"search as characters are entered
set incsearch
if has('nvim')
	set inccommand=nosplit
endif

" highlight search matches
set hlsearch

" context lines above, below, left and right of cursor
set scrolloff=5
set sidescrolloff=5

" Disable Mac/GVim Scroll Bars
set guioptions=

""""""""""""""""""""""""""""" SAVING AND UNDO
" Write before :make
set autowrite

" Persistent undo
set undofile
set undodir=~/.vim/undodir

" Other persistence
" :help 21.3
" :help restore-cursor
function! ResCur()
	if line("'\"") >= 1 && line("'\"") <= line("$") && &ft !~# 'commit'
		exe "normal! g`\""
	endif
endfunction
autocmd BufReadPost * call ResCur()

""""""""""""""""""""""""""""" TABS & INDENT
" Always indent to an indent level with ">>" and "<<"
" Cleans up leading spaces
set shiftround

" default to smaller tabs plz
set tabstop=4 softtabstop=4 shiftwidth=4

""""""""""""""""""""""""""""" FILETYPE SETTINGS
" Python "
" run file
autocmd FileType python nmap <leader>b :w<CR>:!python %<CR>

" GOLANG "
" run :GoBuild
autocmd FileType go nmap <leader>b :GoBuild<CR>
autocmd FileType go nmap <leader>r :GoRun %<CR>
autocmd FileType go setlocal sw=4 sts=4 ts=4
autocmd FileType go let g:go_fmt_command = "goimports"

" YAML "
autocmd FileType yaml setlocal sw=2 sts=2 ts=2 expandtab

" Lua "
autocmd FileType lua setlocal sw=4 sts=4 ts=4 expandtab nowrap

" C++ "
autocmd FileType cpp setlocal sw=3 sts=3 ts=3 expandtab nowrap

" Markdown
au BufNewFile,BufReadPost *.md,*.mkd,*.markdown set filetype=markdown
autocmd FileType markdown setlocal spell spelllang=en_us sw=4 sts=4 ts=4 expandtab wrap
let g:markdown_folding = 1

" Jenkinsfiles
au BufRead,BufNewFile Ghenkinsfile,Hyboriafile,Jenkinsfile set filetype=jenkinsfile
au FileType jenkinsfile setlocal sw=2 sts=2 ts=2 expandtab syntax=groovy

"""""""""""""""""""""""""""" STATUSLINE
set stl=%3c                            " column number
set stl+=\ <%.35f>                     " filename
set stl+=%m                            " modified
set stl+=\ [%l\ \|\ %P]                " line number, file percentage
set stl+=\ %h
set stl+=%=                            " right align
set stl+=\ %y                          " detected filetype
set stl+=%{fugitive#statusline()}
set stl+=%{coc#status()}%{get(b:,'coc_current_function','')}
